! Program to add wcs header to a kappa reconstruction of an optical image
! Assumes the following
! 1) Y axis points North, X axis points West exactly
! 2) (0,0) in the kappa map matches the (0,0) in the image
! 3) The kappa map covers the full image
!----------------------------------------------

Module Main
integer :: unit,status,naxes(1:2)
end Module Main
!----------------------------------------------

Program add_wcs_header
use main
implicit none

integer :: i
character*100 :: image_fitsfile,kappa_fitsfile
character*200 :: keyword(1:10), comment
real*4, dimension(1:10) :: keyval, imkeyval
real*4 :: zero_ra, zero_dec, ra_width, dec_width

character*150 arg,opt
integer iargc,narg

narg=iargc()

do i=1,narg
   call getarg(i,opt)
   call getarg(i+1,arg)
   select case (opt)
   case ('-h')
      print *,'add_wcs_header.a'
      print *,'Example: '
      print *,'add_wcs_header.a -image image.fits -kappa kappa.fits'
      print *,''
      STOP

   case ('-image')
      image_fitsfile=arg
   case ('-kappa')
      kappa_fitsfile=arg
   end select

enddo

comment = ''

!------------------------------------------

! First read the wcs from the original fits image

call openfits(image_fitsfile)  ! reads naxes

keyword(1) = 'CRVAL1'
keyword(2) = 'CRPIX1'
keyword(3) = 'CDELT1'
keyword(4) = 'CRVAL2'
keyword(5) = 'CRPIX2'
keyword(6) = 'CDELT2'

do i = 1,6
   call FTGKYE(unit,keyword(i), imkeyval(i),comment,status)
end do

zero_ra = imkeyval(1) - imkeyval(2)*imkeyval(3)
ra_width = imkeyval(3) * naxes(1)
zero_dec = imkeyval(4) - imkeyval(5)*imkeyval(6)
dec_width = imkeyval(6) * naxes(2)

! close image fits file

call ftflus(unit,status)
call ftclos(unit,status)

!------------------------------------------

! Copy the wcs to the Kappa fits file scaled by the kappa naxes

call openfits(kappa_fitsfile)  ! reads kappa naxes

call FTPKLS(unit,'RADECSYS','ICRS','', status)
call FTPKLS(unit,'CTYPE1','RA--TAN','', status)
call FTPKLS(unit,'CUNIT1','deg','', status)

keyword(1) = 'CRVAL1'
keyword(2) = 'CRPIX1'
keyword(3) = 'CD1_1'
keyword(4) = 'CD1_2'

keyval(1) = zero_ra
keyval(2) = 0.000
keyval(3) = ra_width/naxes(1)
keyval(4) = 0.000

do i = 1,4
   call FTPKYE(unit,keyword(i),keyval(i),5,comment,status)
end do

call FTPKLS(unit,'CTYPE2','DEC--TAN','', status)
call FTPKLS(unit,'CUNIT2','deg','', status)

keyword(1) = 'CRVAL2'
keyword(2) = 'CRPIX2'
keyword(3) = 'CD2_1'
keyword(4) = 'CD2_2'

keyval(1) = zero_dec
keyval(2) = 0.000
keyval(3) = 0.000
keyval(4) = dec_width/naxes(2)

do i = 1,4
   call FTPKYE(unit,keyword(i),keyval(i),5,comment,status)
end do

! Closing Kappa FITS file 
      
call ftflus(unit,status)
call ftclos(unit,status)


end Program add_wcs_header


!***************************************************************
! FITSIO ROUTINES 
!***************************************************************

Subroutine openfits(filename)
use main
implicit none

character(len=100) filename
integer             :: readwrite,blocksize
integer             :: nfound

! The status and file unit parameters initialized.

status = 0      

! Open the read/write FITS file 
! Denoted by region, fieldname and camerano from do loop above
! Note blocksize is anobsolete ignorable parameter

readwrite=1

unit = 99
call ftopen(unit,filename,readwrite,blocksize,status)

! Determine the size of the image.

call ftgknj(unit,'NAXIS',1,2,naxes,nfound,status)

! Check that it found both NAXIS1 and NAXIS2 keywords.

if (nfound .ne. 2)then
   write(*,*) filename
   print *,'READIMAGE failed to read the fits file',filename
   return
end if

end Subroutine openfits
!--------------------------------------------------------
